﻿namespace SnakeWF.Model

//A modellhez szükséges segédosztályok és típusok gyűjteménye.

//Koordináta osztály, a definiált paraméterekkel inicializálhatjuk az objektumunkat külön konstruktor létrehozása nélkül.
type Coordinate(xCoord : int, yCoord : int) =
    class

        //Privát mezők
        let mutable x = xCoord
        let mutable y = yCoord
        
        //Publikus tulajdonságok
        member this.X
            with get() = x
            and set(value) = x <- value

        member this.Y
            with get() = y
            and set(value) = y <- value

        //Többféle paraméterrel ellátott konstruktort is definiálhatunk.
        new() = Coordinate(0, 0)

        //Object metódusok felüldefiniálása
        override this.ToString() =
            this.X.ToString() + " " + this.Y.ToString()

        override this.Equals(other : obj) = 
            match other with
                | null -> nullArg "other"
                | :? Coordinate as coord -> this.Equals(coord)
                | _ -> invalidArg "other" "Must be an instance of Coordinate."

        member this.Equals(other : Coordinate) =
            let dx = this.X - other.X
            let dy = this.Y - other.Y
            dx = 0 && dy = 0

        override this.GetHashCode() =
            pown this.X this.Y 
        
        //Külső kódból az (==) operátort meghívva az op_Equality fog lefutni (ha ez nincs, mindenképp referencia szerinti 
        //összehasonlítást fog végezni a program, attól függetlenül, hogy megvalósítottuk-e az equals metódust, vagy sem)
        static member op_Equality (lhs : Coordinate , rhs : Coordinate) =
            lhs.Equals(rhs)
    end

//Irány felsorolási típus (A különböző felsorolási értékeket a megszokott módon megfeleltethetjük diszkrét értékeknek)
type Direction =
    | North = 1
    | East = 2
    | South = -1
    | West = -2

//Játék vége esemény argumentumai
type GameOverEventArgs(reachedScore : int) =
    class
        inherit System.EventArgs() //Öröklés a System.EventArgs osztályból

        member this.Score = reachedScore
    end

//Nézet frissítése esemény argumentumai
type UpdateEventArgs(food : Coordinate, snake : System.Collections.IEnumerable) =
    class
        inherit System.EventArgs()

        member this.Food = food
        member this.Snake = snake
    end
