﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SnakeWF.Model;
using System.Diagnostics;

namespace SnakeWF.View
{
    public partial class SnakeView : Form
    {
        private Boolean _isPaused = false;

        SnakeWF.Model.SnakeModel gameModel;
        Graphics graphics;

        private Bitmap _drawBitmap;
        public SnakeView()
        {
            InitializeComponent();
            
            gameModel = new SnakeWF.Model.SnakeModel();

            this.KeyPreview = true;
            this.KeyDown += SnakeView_KeyDown;

            gameModel.Update += RefreshTable;
            gameModel.GameOver += Game_GameOver;

            _drawBitmap = new Bitmap(_Panel.Width,_Panel.Height);
        }


        void SnakeView_KeyDown(object sender, KeyEventArgs e)
        {
            if (_isPaused)
                return;

            if (e.KeyCode == Keys.F2)
                gameModel.NewGame();

            if (e.KeyCode == Keys.F10)
                Application.Exit();

            if (!gameModel.isGameOn)
                return;

            switch (e.KeyCode)
            {
                case Keys.Up:
                    gameModel.Move(Direction.North);
                    break;
                case Keys.Down:
                    gameModel.Move(Direction.South);
                    break;
                case Keys.Right:
                    gameModel.Move(Direction.East);
                    break;
                case Keys.Left:
                    gameModel.Move(Direction.West);
                    break;
            }
        }

        void Game_GameOver(object sender, Model.GameOverEventArgs e)
        {
            if (gameModel.isGameOn)
                MessageBox.Show("Játék vége, az Ön pontja: " + e.Score);
        }

        private void newGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            gameModel.NewGame();
        }

        private void kilépésToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void RefreshTable(object sender, UpdateEventArgs e)
        {
            
            if(graphics == null)
                graphics = Graphics.FromImage(_drawBitmap);

            lock (graphics)
            {
                try
                {
                    graphics.Clear(Color.White);
                    Int32 actFieldSizeX = _Panel.Width / gameModel.TableSize;
                    Int32 actFieldSizeY = _Panel.Height / gameModel.TableSize;

                    foreach (Coordinate coord in e.Snake)
                        graphics.FillRectangle(Brushes.Black, coord.X * actFieldSizeX, coord.Y * actFieldSizeY, actFieldSizeX, actFieldSizeY);

                    graphics.DrawEllipse(new Pen(Color.Black), e.Food.X * actFieldSizeX, e.Food.Y * actFieldSizeY, actFieldSizeX, actFieldSizeY);
                    _Panel.CreateGraphics().DrawImage(_drawBitmap, 0, 0);
                }
                catch(Exception) { }
            }



        }

        private void szünetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            gameModel.PauseOrResume();
            _isPaused = !_isPaused;
        }


    }
}
